// UpdatingGUI2.java
// Eliminating Race Conditions when updating Swing Components.
/*
Swing Event Dispatch Thread:

loop
   wait for interaction with the GUI;
   on interaction:
        dispatch appropriate event; // (1)
endloop

Comment to code at (1) : corresponding program action is executed in the event dispatch thread.
Program action should not hoard the event dispatch thread, but should create a new thread if the action is computation intensive.

Swing components are not thread safe.
- other threads can change the properties in the GUI.

Recommended solution for changing the GUI properties from other threads:
- use either SwingUtilities.invokeLater() method or SwingUtilities.invokeAndWait() method.
- both methods take a Runnable object as an argument.
- the run() method of the Runnable object can include code to update the GUI.
- the run() method of the Runnable object is executed in the event dispatch thread after other pending events have been dealt with.
*/
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class UpdatingGUI2 extends JFrame {
  private JTextField infoField;

  public UpdatingGUI2() {
    Container cp = getContentPane();
    infoField = new JTextField(" One");
    cp.add(infoField, BorderLayout.NORTH);

    addWindowListener(new WindowAdapter() {
      public void windowOpened(WindowEvent e) { // Executed in the event dispatch thread
        try { // Intentional initialization overhead
          Thread.sleep(2000);
        } catch (InterruptedException ex) {
          System.err.println(ex);
        }
        infoField.setText(infoField.getText() + " Two");
      }
    });

    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    pack();
    setVisible(true);
  }

  public static void main(String[] args) throws Exception {
    final UpdatingGUI2 frame = new UpdatingGUI2();
    // SwingUtilities.invokeAndWait(new Runnable() {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() { // Executed in the event dispatch thread
        frame.infoField.setText(frame.infoField.getText() + " Three");
      }
    });
    System.out.println("Four");
  }
}